/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import java.io.Closeable;
import java.io.IOException;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
public class CloseableIterator<T> implements Iterator<T>, Closeable {
    public static <T, U extends Iterator<T> & Closeable> CloseableIterator<T> newInstance(U value) {
        return new CloseableIterator<T>(value, value);
    }

    private final Iterator<T> iterator;
    private final Closeable closeable;
    public CloseableIterator(Iterator<T> iterator, Closeable closeable) {
        this.iterator = iterator;
        this.closeable = closeable;
    }

    @Override
    public void close() {
        try {
            closeable.close();
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public boolean hasNext() {
        return iterator.hasNext();
    }

    @Override
    public T next() {
        return iterator.next();
    }

    @Override
    public void remove() {
        iterator.remove();
    }

    public CloseableIterator<T> additionallyClosing(Closeable addtionalCloseable) {
        return CloseableIterators.additionallyClosing(this, addtionalCloseable);
    }

    public CloseableIterator<T> withAdditionalClosingAction(Runnable action) {
        return CloseableIterators.withAdditionalClosingAction(this, action);
    }
}
