/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class FileBytesStreamBehaviour implements StreamBehaviour<Byte> {
    private final File file;

    public FileBytesStreamBehaviour(File file) {
        this.file = file;
    }

    @Override
    public CloseableIterator<Byte> openCloseableIterator() {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                CloseableIterator<Byte> result = CloseableIterators.bytes(inputStream);
                inputStream = null;
                return result;
            } finally {
                if (inputStream != null)
                    inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public void forEach(Consumer<? super Byte> consumer) {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                int singleByte;
                while ((singleByte = inputStream.read()) >= 0) {
                    consumer.accept((byte)singleByte);
                }
            } finally {
                inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

}
