/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class FileCharactersStreamBehaviour implements StreamBehaviour<Character> {
    private final File file;
    private final Charset charset;

    public FileCharactersStreamBehaviour(File file, Charset charset) {
        this.file = file;
        this.charset = charset;
    }

    @Override
    public CloseableIterator<Character> openCloseableIterator() {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
                CloseableIterator<Character> result = CloseableIterators.characters(bufferedReader);
                inputStream = null;
                return result;
            } finally {
                if (inputStream != null)
                    inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public void forEach(Consumer<? super Character> consumer) {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
                int character;
                while ((character = bufferedReader.read()) >= 0) {
                    consumer.accept((char)character);
                }
            } finally {
                inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

}
