/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.NoSuchElementException;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class FileLinesStreamBehaviour implements StreamBehaviour<String> {
    private final File file;
    private final Charset charset;

    public FileLinesStreamBehaviour(File file, Charset charset) {
        this.file = file;
        this.charset = charset;
    }

    @Override
    public CloseableIterator<String> openCloseableIterator() {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
                CloseableIterator<String> result = CloseableIterators.lines(bufferedReader);
                inputStream = null;
                return result;
            } finally {
                if (inputStream != null)
                    inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public void forEach(Consumer<? super String> consumer) {
        try {
            InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
            try {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    consumer.accept(line);
                }
            } finally {
                inputStream.close();
            }
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

}
