/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;
import com.github.sviperll.UnaryOperator;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class IteratedStreamBehaviour<T> implements StreamBehaviour<T> {
    private final T seed;
    private final UnaryOperator<T> function;

    public IteratedStreamBehaviour(T seed, UnaryOperator<T> function) {
        this.seed = seed;
        this.function = function;
    }

    @Override
    public CloseableIterator<T> openCloseableIterator() {
        return CloseableIterators.iterator(new Iterator<T>() {
            private T nextValue = seed;

            @Override
            public boolean hasNext() {
                return true;
            }

            @Override
            public T next() {
                T result = nextValue;
                nextValue = function.apply(nextValue);
                return result;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException("Not supported.");
            }
        });
    }

    @Override
    public void forEach(Consumer<? super T> consumer) {
        T value = seed;
        for (;;) {
            consumer.accept(value);
            value = function.apply(value);
        }
    }

}
