/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;
import com.github.sviperll.Function;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class MappedStreamBehaviour<T, U> implements StreamBehaviour<U> {
    private final StreamBehaviour<T> upstream;
    private final Function<? super T, ? extends U> function;
    public MappedStreamBehaviour(StreamBehaviour<T> upstream, Function<? super T, ? extends U> function) {
        this.upstream = upstream;
        this.function = function;
    }

    @Override
    public CloseableIterator<U> openCloseableIterator() {
        final CloseableIterator<T> upstreamIterator = upstream.openCloseableIterator();
        Iterator<U> iterator = new Iterator<U>() {
            @Override
            public boolean hasNext() {
                return upstreamIterator.hasNext();
            }

            @Override
            public U next() {
                return function.apply(upstreamIterator.next());
            }

            @Override
            public void remove() {
                upstreamIterator.remove();
            }
        };
        return new CloseableIterator<U>(iterator, upstreamIterator);
    }

    @Override
    public void forEach(final Consumer<? super U> consumer) {
        upstream.forEach(new Consumer<T>() {
            @Override
            public void accept(T value) {
                consumer.accept(function.apply(value));
            }
        });
    }
}
