/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.Consumer;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class SkippedStreamBehaviour<T> implements StreamBehaviour<T> {
    private final StreamBehaviour<T> stream;
    private final int offset;
    public SkippedStreamBehaviour(final StreamBehaviour<T> stream, final int offset) {
        this.stream = stream;
        this.offset = offset;
    }

    @Override
    public CloseableIterator<T> openCloseableIterator() {
        CloseableIterator<T> iterator = stream.openCloseableIterator();
        try {
            for (int i = 0; i < offset && iterator.hasNext(); i++)
                iterator.next();
            CloseableIterator<T> result = iterator;
            iterator = null;
            return result;
        } finally {
            if (iterator != null)
                return iterator;
        }
    }

    @Override
    public void forEach(final Consumer<? super T> consumer) {
        stream.forEach(new Consumer<T>() {
            int count = 0;
            @Override
            public void accept(T value) {
                if (count >= offset)
                    consumer.accept(value);
                count++;
            }
        });
    }
}
