/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.BiFunction;
import com.github.sviperll.BinaryOperator;
import com.github.sviperll.Consumer;
import com.github.sviperll.Function;
import com.github.sviperll.Generator;
import com.github.sviperll.OptionalVisitor;
import com.github.sviperll.Predicate;
import com.github.sviperll.Supplier;
import org.github.sviperll.stream.Collector.CollectorState;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
public class Stream<T> implements StreamBehaviour<T> {
    private final StreamBehaviour<T> behaviour;
    public Stream(StreamBehaviour<T> behaviour) {
        this.behaviour = behaviour;
    }

    @Override
    public CloseableIterator<T> openCloseableIterator() {
        return behaviour.openCloseableIterator();
    }

    @Override
    public void forEach(Consumer<? super T> consumer) {
        behaviour.forEach(consumer);
    }

    public <U> Stream<U> map(Function<? super T, U> function) {
        return Streams.map(behaviour, function);
    }

    public Stream<T> filter(Predicate<? super T> predicate) {
        return Streams.filter(behaviour, predicate);
    }

    public <U> U reduce(final U seed, final BiFunction<U, ? super T, U> function) {
        return Streams.reduce(behaviour, seed, function);
    }

    public <R, E extends Exception> R reduce(final BinaryOperator<T> operator, OptionalVisitor<? super T, R, E> visitor) throws E {
        return Streams.reduce(behaviour, operator, visitor);
    }

    public <R> Stream<R> flatMap(Function<? super T, ? extends Generator<R>> function) {
        return Streams.flatMap(behaviour, function);
    }

    public Stream<T> skip(int offset) {
        return Streams.skip(behaviour, offset);
    }

    public Stream<T> limit(int size) {
        return Streams.limit(behaviour, size);
    }

    public <R, E extends Exception> R collect(Supplier<? extends CollectorState<? super T, R, E>> collector) throws E {
        return Streams.collect(behaviour, collector);
    }

    public <R, E extends Exception> R findFirst(OptionalVisitor<? super T, R, E> visitor) throws E {
        return Streams.findFirst(behaviour, visitor);
    }

    public Stream<T> withAdditionalClosingAction(Runnable action) {
        return Streams.withAdditionalClosingAction(behaviour, action);
    }

    public int count() {
        return Streams.count(behaviour);
    }
}
